-- Test add_invoice
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(16);

set search_path to auth, camper, public;

select has_function('camper', 'add_invoice', array ['integer', 'date', 'integer', 'text', 'integer', 'new_invoice_product[]']);
select function_lang_is('camper', 'add_invoice', array ['integer', 'date', 'integer', 'text', 'integer', 'new_invoice_product[]'], 'plpgsql');
select function_returns('camper', 'add_invoice', array ['integer', 'date', 'integer', 'text', 'integer', 'new_invoice_product[]'], 'uuid');
select isnt_definer('camper', 'add_invoice', array ['integer', 'date', 'integer', 'text', 'integer', 'new_invoice_product[]']);
select volatility_is('camper', 'add_invoice', array ['integer', 'date', 'integer', 'text', 'integer', 'new_invoice_product[]'], 'volatile');
select function_privs_are('camper', 'add_invoice', array ['integer', 'date', 'integer', 'text', 'integer', 'new_invoice_product[]'], 'guest', array []::text[]);
select function_privs_are('camper', 'add_invoice', array ['integer', 'date', 'integer', 'text', 'integer', 'new_invoice_product[]'], 'employee', array ['EXECUTE']);
select function_privs_are('camper', 'add_invoice', array ['integer', 'date', 'integer', 'text', 'integer', 'new_invoice_product[]'], 'admin', array ['EXECUTE']);
select function_privs_are('camper', 'add_invoice', array ['integer', 'date', 'integer', 'text', 'integer', 'new_invoice_product[]'], 'authenticator', array []::text[]);


set client_min_messages to warning;
truncate invoice_number_counter cascade;
truncate invoice_product_tax cascade;
truncate invoice_product cascade;
truncate invoice cascade;
truncate contact cascade;
truncate product cascade;
truncate tax cascade;
truncate tax_class cascade;
truncate payment_method cascade;
truncate company cascade;
reset client_min_messages;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, rtc_number, tourist_tax, tourist_tax_max_days, country_code, currency_code, default_lang_tag, invoice_number_format)
values (1, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', '', 60, 7, 'ES', 'EUR', 'ca', '"F"YYYY0000')
     , (2, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', '', 60, 7, 'FR', 'USD', 'ca', '"INV"000-YY')
;

insert into payment_method (payment_method_id, company_id, name, instructions)
values (111, 1, 'cash', 'cash')
     , (222, 2, 'cash', 'cash')
;

insert into invoice_number_counter (company_id, year, currval)
values (1, 2023, '5')
     , (2, 2023, '55')
;

insert into tax_class (tax_class_id, company_id, name)
values (11, 1, 'tax')
     , (22, 2, 'tax')
;

insert into tax (tax_id, company_id, tax_class_id, name, rate)
values (3, 1, 11, 'IRPF -15 %', -0.15)
     , (4, 1, 11, 'IVA 21 %', 0.21)
     , (5, 2, 22, 'IRPF -7 %', -0.07)
     , (6, 2, 22, 'IVA 10 %', 0.10)
;

insert into contact (contact_id, company_id, name, id_document_type_id, id_document_number, address, city, province, postal_code, country_code)
values (12, 1, 'Contact 2.1', 'D', '41440443Q', 'Fake St', 'City', 'Province', '17600', 'ES')
     , (13, 1, 'Contact 2.2', 'D', '41440443Q', 'Fake St', 'City', 'Province', '17600', 'ES')
     , (14, 2, 'Contact 4.1', 'D', '41440443Q', 'Fake St', 'City', 'Province', '17600', 'ES')
     , (15, 2, 'Contact 4.2', 'D', '41440443Q', 'Fake St', 'City', 'Province', '17600', 'ES')
;

insert into product (product_id, company_id, name, price)
values ( 7, 1, 'Product 2.1', 1212)
     , ( 8, 1, 'Product 2.2', 2424)
     , ( 9, 2, 'Product 4.1', 4848)
     , (10, 2, 'Product 4.2', 9696)
     , (11, 2, 'Product 4.3', 1010)
;

select lives_ok(
	$$ select add_invoice(1, '2023-02-15', 12, 'Notes 1', 111, '{"(7,Product 1,Description 1,12.24,2,0.0,{4})"}') $$,
	'Should be able to insert an invoice for the first company with a product'
);

select lives_ok(
	$$ select add_invoice(1, '2023-02-16', 13, 'Notes 2', 111, '{"(7,Product 1 bis,Description 1 bis,33.33,1,0.50,\"{4,3}\")","(8,Product 2,Description 2,24.00,3,0.75,{})"}') $$,
	'Should be able to insert a second invoice for the first company with two product'
);

select lives_ok(
	$$ select add_invoice(2, '2023-02-14', 15, 'Notes 3', 222, '{"(11,Product 4.3,,11.11,1,0.0,{6})","(,Product 4.4,Description 4.4,22.22,3,0.05,{})"}') $$,
	'Should be able to insert an invoice for the second company with a product'
);

select bag_eq(
	$$ select company_id, invoice_number, invoice_date, contact_id, invoice_status, notes, payment_method_id, currency_code, created_at from invoice $$,
	$$ values (1, 'F20230006', '2023-02-15'::date, 12, 'created', 'Notes 1', 111, 'EUR', current_timestamp)
	        , (1, 'F20230007', '2023-02-16'::date, 13, 'created', 'Notes 2', 111, 'EUR', current_timestamp)
	        , (2, 'INV056-23', '2023-02-14'::date, 15, 'created', 'Notes 3', 222, 'USD', current_timestamp)
	$$,
	'Should have created all invoices'
);

select bag_eq(
	$$ select invoice_number, name, description, price, quantity, discount_rate from invoice_product join invoice using (invoice_id) $$,
	$$ values ('F20230006', 'Product 1', 'Description 1', 1224, 2, 0.00)
	        , ('F20230007', 'Product 1 bis', 'Description 1 bis', 3333, 1, 0.50)
	        , ('F20230007', 'Product 2', 'Description 2', 2400, 3, 0.75)
	        , ('INV056-23', 'Product 4.3', '', 1111, 1, 0.0)
	        , ('INV056-23', 'Product 4.4', 'Description 4.4', 2222, 3, 0.05)
	$$,
	'Should have created all invoice products'
);

select bag_eq(
	$$ select invoice_number, product_id, name from invoice_product left join invoice_product_product using (invoice_product_id) join invoice using (invoice_id) $$,
	$$ values ('F20230006', 7, 'Product 1')
	        , ('F20230007', 7, 'Product 1 bis')
	        , ('F20230007', 8, 'Product 2')
	        , ('INV056-23', 11, 'Product 4.3')
	        , ('INV056-23', NULL, 'Product 4.4')
	$$,
	'Should have linked all invoice products'
);

select bag_eq(
	$$ select invoice_number, name, tax_id, tax_rate from invoice_product_tax join invoice_product using (invoice_product_id) join invoice using (invoice_id) $$,
	$$ values ('F20230006', 'Product 1', 4, 0.21)
	        , ('F20230007', 'Product 1 bis', 4, 0.21)
	        , ('F20230007', 'Product 1 bis', 3, -0.15)
	        , ('INV056-23', 'Product 4.3', 6, 0.10)
	$$,
	'Should have created all invoice product taxes'
);


select *
from finish();

rollback;
