-- Test invoice
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(72);

set search_path to camper, auth, public;

select has_table('invoice');
select has_pk('invoice' );
select table_privs_are('invoice', 'guest', array []::text[]);
select table_privs_are('invoice', 'employee', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('invoice', 'admin', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('invoice', 'authenticator', array []::text[]);

select has_column('invoice', 'invoice_id');
select col_is_pk('invoice', 'invoice_id');
select col_type_is('invoice', 'invoice_id', 'integer');
select col_not_null('invoice', 'invoice_id');
select col_hasnt_default('invoice', 'invoice_id');

select has_column('invoice', 'company_id');
select col_is_fk('invoice', 'company_id');
select fk_ok('invoice', 'company_id', 'company', 'company_id');
select col_type_is('invoice', 'company_id', 'integer');
select col_not_null('invoice', 'company_id');
select col_hasnt_default('invoice', 'company_id');

select has_column('invoice', 'slug');
select col_is_unique('invoice', 'slug');
select col_type_is('invoice', 'slug', 'uuid');
select col_not_null('invoice', 'slug');
select col_has_default('invoice', 'slug');
select col_default_is('invoice', 'slug', 'gen_random_uuid()');

select has_column('invoice', 'invoice_number');
select col_type_is('invoice', 'invoice_number', 'text');
select col_not_null('invoice', 'invoice_number');
select col_hasnt_default('invoice', 'invoice_number');

select has_column('invoice', 'invoice_date');
select col_type_is('invoice', 'invoice_date', 'date');
select col_not_null('invoice', 'invoice_date');
select col_has_default('invoice', 'invoice_date');
select col_default_is('invoice', 'invoice_date', 'CURRENT_DATE');

select has_column('invoice', 'contact_id');
select col_is_fk('invoice', 'contact_id');
select fk_ok('invoice', 'contact_id', 'contact', 'contact_id');
select col_type_is('invoice', 'contact_id', 'integer');
select col_not_null('invoice', 'contact_id');
select col_hasnt_default('invoice', 'contact_id');

select has_column('invoice', 'invoice_status');
select col_is_fk('invoice', 'invoice_status');
select fk_ok('invoice', 'invoice_status', 'invoice_status', 'invoice_status');
select col_type_is('invoice', 'invoice_status', 'text');
select col_not_null('invoice', 'invoice_status');
select col_has_default('invoice', 'invoice_status');
select col_default_is('invoice', 'invoice_status', 'created');

select has_column('invoice', 'notes');
select col_type_is('invoice', 'notes', 'text');
select col_not_null('invoice', 'notes');
select col_has_default('invoice', 'notes');
select col_default_is('invoice', 'notes', '');

select has_column('invoice', 'payment_method_id');
select col_is_fk('invoice', 'payment_method_id');
select fk_ok('invoice', 'payment_method_id', 'payment_method', 'payment_method_id');
select col_type_is('invoice', 'payment_method_id', 'integer');
select col_not_null('invoice', 'payment_method_id');
select col_hasnt_default('invoice', 'payment_method_id');

select has_column('invoice', 'currency_code');
select col_is_fk('invoice', 'currency_code');
select fk_ok('invoice', 'currency_code', 'currency', 'currency_code');
select col_type_is('invoice', 'currency_code', 'text');
select col_not_null('invoice', 'currency_code');
select col_hasnt_default('invoice', 'currency_code');

select has_column('invoice', 'created_at');
select col_type_is('invoice', 'created_at', 'timestamp with time zone');
select col_not_null('invoice', 'created_at');
select col_has_default('invoice', 'created_at');
select col_default_is('invoice', 'created_at', 'CURRENT_TIMESTAMP');


set client_min_messages to warning;
truncate invoice cascade;
truncate contact cascade;
truncate payment_method cascade;
truncate company_host cascade;
truncate company_user cascade;
truncate company cascade;
truncate payment_method cascade;
truncate auth."user" cascade;
reset client_min_messages;

insert into auth."user" (user_id, email, name, password, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, rtc_number, tourist_tax, tourist_tax_max_days, country_code, currency_code, default_lang_tag)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', '', 60, 7, 'ES', 'EUR', 'ca')
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', '', 60, 7, 'FR', 'USD', 'ca')
;

insert into company_user (company_id, user_id, role)
values (2, 1, 'admin')
     , (4, 5, 'admin')
;

insert into company_host (company_id, host)
values (2, 'co2')
     , (4, 'co4')
;

insert into payment_method (payment_method_id, company_id, name, instructions)
values (444, 4, 'cash', 'cash')
     , (222, 2, 'cash', 'cash')
;

insert into contact (contact_id, company_id, name, id_document_type_id, id_document_number, address, city, province, postal_code, country_code)
values (6, 2, 'Contact 1', 'D', '41440443Q', 'Fake St', 'City', 'Province', '17600', 'ES')
     , (8, 4, 'Contact 2', 'D', '41440443Q', 'Fake St', 'City', 'Province', '17600', 'ES')
;

insert into invoice (company_id, invoice_number, contact_id, currency_code, payment_method_id)
values (2, 'INV020001', 6, 'EUR', 222)
     , (4, 'INV040001', 8, 'EUR', 444)
;


prepare invoice_data as
select company_id, invoice_number
from invoice
order by company_id, invoice_number;

set role employee;
select is_empty('invoice_data', 'Should show no data when cookie is not set yet');
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog', 'co2');
select bag_eq(
	'invoice_data',
	$$ values (2, 'INV020001')
	$$,
	'Should only list invoices of the companies where demo@tandem.blog is user of'
);
reset role;

select set_cookie('12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524/admin@tandem.blog', 'co4');
select bag_eq(
	'invoice_data',
	$$ values (4, 'INV040001')
	$$,
	'Should only list invoices of the companies where admin@tandem.blog is user of'
);
reset role;

select set_cookie('not-a-cookie', 'co4');
select throws_ok(
	'invoice_data',
	'42501', 'permission denied for table invoice',
	'Should not allow select to guest users'
);
reset role;


select throws_ok( $$
	insert into invoice (company_id, invoice_number, contact_id, currency_code, payment_method_id)
	values (2, '  ', 6, 'EUR', 222)
	$$,
       '23514', 'new row for relation "invoice" violates check constraint "invoice_number_not_empty"',
	'Should not allow invoice with blank number'
);


select *
from finish();

rollback;

