-- Test tax_class
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(26);

set search_path to camper, auth, public;

select has_table('tax_class');
select has_pk('tax_class' );
select table_privs_are('tax_class', 'guest', array []::text[]);
select table_privs_are('tax_class', 'employee', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('tax_class', 'admin', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('tax_class', 'authenticator', array []::text[]);

select has_column('tax_class', 'tax_class_id');
select col_is_pk('tax_class', 'tax_class_id');
select col_type_is('tax_class', 'tax_class_id', 'integer');
select col_not_null('tax_class', 'tax_class_id');
select col_hasnt_default('tax_class', 'tax_class_id');

select has_column('tax_class', 'company_id');
select col_is_fk('tax_class', 'company_id');
select fk_ok('tax_class', 'company_id', 'company', 'company_id');
select col_type_is('tax_class', 'company_id', 'integer');
select col_not_null('tax_class', 'company_id');
select col_hasnt_default('tax_class', 'company_id');

select has_column('tax_class', 'name');
select col_type_is('tax_class', 'name', 'text');
select col_not_null('tax_class', 'name');
select col_hasnt_default('tax_class', 'name');


set client_min_messages to warning;
truncate tax_class cascade;
truncate company_host cascade;
truncate company_user cascade;
truncate company cascade;
truncate auth."user" cascade;
reset client_min_messages;

insert into auth."user" (user_id, email, name, password, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, rtc_number, tourist_tax, tourist_tax_max_days, country_code, currency_code, default_lang_tag)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', '', 60, 7, 'ES', 'EUR', 'ca')
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', '', 60, 7, 'FR', 'USD', 'ca')
;

insert into company_user (company_id, user_id, role)
values (2, 1, 'admin')
     , (4, 5, 'admin')
;

insert into company_host (company_id, host)
values (2, 'co2')
     , (4, 'co4')
;

insert into tax_class (company_id, name)
values (2, 'VAT')
     , (2, 'IRPF')
     , (4, 'VAT')
     , (4, 'import')
;

prepare tax_class_data as
select company_id, name
from tax_class
order by company_id;

set role employee;
select is_empty('tax_class_data', 'Should show no data when cookie is not set yet');
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog', 'co2');
select bag_eq(
	'tax_class_data',
	$$ values (2, 'VAT')
	        , (2, 'IRPF')
	$$,
	'Should only list taxes of the companies where demo@tandem.blog is user of'
);
reset role;

select set_cookie('12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524/admin@tandem.blog', 'co4');
select bag_eq(
	'tax_class_data',
	$$ values (4, 'VAT')
	        , (4, 'import')
	$$,
	'Should only list taxes of the companies where admin@tandem.blog is user of'
);
reset role;

select set_cookie('not-a-cookie', 'co2');
select throws_ok(
	'tax_class_data',
	'42501', 'permission denied for table tax_class',
	'Should not allow select to guest users'
);
reset role;

select throws_ok( $$
	insert into tax_class (company_id, name)
	values (2, '  ')
	$$,
	'23514', 'new row for relation "tax_class" violates check constraint "name_not_empty"',
	'Should not allow classes with blank name'
);


select *
from finish();

rollback;

